// SPDX-FileCopyrightText: 2020 - 2023 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: GPL-3.0-or-later

#ifndef DFMIO_GLOBAL_H
#define DFMIO_GLOBAL_H

//  namespace
#define DFMIO dfmio
#define BEGIN_IO_NAMESPACE namespace DFMIO {
#define USING_IO_NAMESPACE using namespace DFMIO;
#define END_IO_NAMESPACE }

#endif   // DFMIO_GLOBAL_H
